<?php

App::uses('AppBehavior', 'Model/Behavior');

class CalculateTaxBehavior extends AppBehavior {

	/**
	 * link the model that has this behavior with the tax levels model
	 * so we can contain data
	 */
	public function linkTaxCalc(Model $Model) {
		$Model->bindModel(
			array(
				'belongsTo' => array(
					'TaxLevel' => array(
						'className' => 'EvTax.TaxLevel'
					)
				)
			),
			false
		);
	}

	/**
	 * Initialise the behavior - store the settings against the model
	 *
	 * @see ModelBehavior::setup()
	 */
	public function setup(Model $Model, $settings = array()) {
		$defaults = array(
			'fields' => array(),
			'TaxRateModel' => false,
			'TaxRateModelId' => false
		);

		$this->settings[$Model->alias] = array_merge(
			$defaults,
			$settings
		);

		if (empty($this->settings[$Model->alias]['TaxRateModel'])) {
			$this->linkTaxCalc($Model);
		}
	}

	/**
	 * beforeFind can be used to cancel find operations, or modify the query that will be executed.
	 * By returning null/false you can abort a find. By returning an array you can modify/replace the query
	 * that is going to be run.
	 *
	 * @param Model $model Model using this behavior
	 * @param array $query Data used to execute this query, i.e. conditions, order, etc.
	 * @return bool|array False or null will abort the operation. You can return an array to replace the
	 *   $query that will be eventually run.
	 */
	public function beforeFind(Model $Model, $query) {
		if (empty($this->settings[$Model->alias]['TaxRateModel'])) {
			if (! isset($query['contain']) || (! isset($query['contain']['TaxLevel']) && array_search('TaxLevel', $query['contain']) === false)) {
				$query['contain'][] = 'TaxLevel';
			}
		}

		return $query;
	}

	/**
	 * After find callback. Can be used to modify any results returned by find.
	 *
	 * @param Model $model Model using this behavior
	 * @param mixed $results The results of the find operation
	 * @param bool $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed An array value will replace the value of $results - any other value will be ignored.
	 */
	public function afterFind(Model $Model, $results, $primary = false) {
		if (empty($this->settings[$Model->alias]['fields']) || ! is_array($this->settings[$Model->alias]['fields'])) {
			return $results;
		}

		$TaxRateModel = null;
		if (! empty($this->settings[$Model->alias]['TaxRateModel'])) {
			$TaxRateModel = EvClassRegistry::init($this->settings[$Model->alias]['TaxRateModel']);
		}

		foreach ($results as $key => $item) {
			$results[$key] = $this->calculateTax($Model, $item, $TaxRateModel);
		}

		return $results;
	}

	/**
	 * calculate the taxed amounts for the item
	 *
	 * @param 	Model  	$model  		Model using this behavior
	 * @param 	array 	$item 			The item we wish to calculate tax for
	 * @param 	object 	$TaxRateModel 	The model from which we can get the tax rate
	 * @return 	array 	$item 			The item with additiona _incTax fields
	 */
	public function calculateTax(Model $Model, $item, $TaxRateModel = null) {
		// try to work out the array format
		if (! isset($item[$Model->alias][0]['id']) && isset($item[$Model->alias]['id'])) {

			$newItem[$Model->alias][0] = $item;
			$item = $newItem;
			$formatChanged = true;
		}

		if (isset($item[$Model->alias]) && !empty($item[$Model->alias])) {
			foreach ($item[$Model->alias] as $key => $pricing) {

				// Work out tax level
				if (isset($pricing['TaxLevel']['rate']) && ! empty($pricing['TaxLevel']['rate'])) {
					$rate = $pricing['TaxLevel']['rate'];
				} elseif (isset($TaxRateModel)) {
					$rate = $this->_getRate($Model, $TaxRateModel, $item);
				} else {
					$rate = 0.00;
				}

				foreach ($this->settings[$Model->alias]['fields'] as $field) {

					if (! array_key_exists($field, $item[$Model->alias][$key])) {
						continue;
					}

					// $item[$Model->alias][$key][$field . '_incTax'] = $item[$Model->alias][$key][$field] + ($item[$Model->alias][$key][$field] * ($rate / 100));
					$item[$Model->alias][$key][$field . '_incTax'] = addTaxToPrice($item[$Model->alias][$key][$field], $rate);
				}
			}
		}

		if (isset($formatChanged) && $formatChanged === true) {
			return $item[$Model->alias];
		} else {
			return $item;
		}
	}

	public function addTaxToPrice($Model, $price, $rate = 0.00) {
		return $price + ($price * ($rate / 100));
	}

	/**
	 * work out the tax rate
	 * - either query table
	 * - retrieve from stored array
	 *
	 * @param 	object 			Model object of the original model that called this behavior
	 * @param 	object 			Model Object of the related model that has the tax rate id
	 * @param 	array 			data array
	 * @return 	decimal 		decimal rate, will return zero if none found
	 */
	protected function _getRate($Model, $TaxRateModel, $item) {
		$modelId = Hash::get($item, $this->settings[$Model->alias]['TaxRateModelId']);

		if (! empty($this->settings[$Model->alias]['taxRateCache'][$modelId])) {
			return $this->settings[$Model->alias]['taxRateCache'][$modelId];
		}

		if (! empty($modelId)) {
			$rate = $TaxRateModel->getTaxRate($modelId);

			if (! empty($rate)) {
				$this->settings[$Model->alias]['taxRateCache'][$modelId] = $rate;
				return $rate;
			}
		}

		return intval(0);
	}
}
